#!/usr/bin/python3

import time
import sys
import redis

from argparse import ArgumentParser

# own libs
from modules.cli_tool.padavan import Padavan
from libraries.exceptions.auraexceptions import PlaylistException
from libraries.base.common import AuraCommon


class Guru(AuraCommon):

    # ------------------------------------------------------------------------------------------ #
    def __init__(self):
        AuraCommon.__init__(self, name="AuraGuru")

#        self.create_parser()
        args = self.init_argument_parser()
        self.handle_arguments(args)

    def handle_arguments(self, args):
        if args.stoptime:
            start = time.time()

        if not args.quiet:
            print("Guru thinking...")

        try:
            p = Padavan(args, self.config)
            p.meditate()
        except PlaylistException as pe:
            # typically there is no next file found
            if not args.quiet:
                print(pe)
            else:
                print("")
            exit(4)
        except redis.exceptions.TimeoutError as te:
            print("Timeout when waiting for redis message. Is AURA daemon running? Exiting...")
            exit(3)

        if not args.quiet:
            print("...result: ")

        if p.stringreply != "":
            if p.stringreply[len(p.stringreply)-1] == "\n":
                print(p.stringreply[0:len(p.stringreply) - 1])
            else:
                print(p.stringreply[0:len(p.stringreply)])

        if args.stoptime:
            end = time.time()
            exectime = end-start
            print("execution time: "+str(exectime)+"s")

    def init_argument_parser(self):
        parser = None
        try:
            parser = self.create_parser()
            args = parser.parse_args()

        except ValueError as e:
            if parser is not None:
                parser.print_help()
            print()
            print(e)
            exit(1)
        except TypeError as e:
            if parser is not None:
                parser.print_help()
            print()
            print(e)
            exit(2)

        return args

    def create_parser(self):
        parser = ArgumentParser()

        # options
        parser.add_argument("-sep", "--stop-execution-time", action="store_true", dest="stoptime", default=False,
                            help="Prints the execution time at the end of the skript")
        parser.add_argument("-q", "--quiet", action="store_true", dest="quiet", default=False,
                            help="Just the result will outputed to stout")

        # getter
        parser.add_argument("-gam", "--get-active-mixer", action="store_true", dest="get_active_mixer", default=False,
                            help="Which mixer is activated?")
        parser.add_argument("-pms", "--print-mixer-status", action="store_true", dest="get_mixer_status", default=False,
                            help="Prints all mixer sources and their states")
        parser.add_argument("-pap", "--print-act-programme", action="store_true", dest="get_act_programme",
                            default=False,
                            help="Prints the actual Programme, the controller holds")

        # liquid manipulation
        parser.add_argument("-am", "--select-mixer", action="store", dest="select_mixer", default=-1,
                            metavar="MIXERNUM",
                            help="Which mixer should be activated?", type=int)

        parser.add_argument("-dm", "--de-select-mixer", action="store", dest="deselect_mixer", default=-1,
                            metavar="MIXERNUM",
                            help="Which mixer should be activated?", type=int)

        parser.add_argument("-vm", "--volume", action="store", dest="set_volume", default=0,
                            metavar=("MIXERNUM", "VOLUME"), nargs=2,
                            help="Set volume of a mixer source", type=int)

        # parser.add_argument("-as", "--add-source", action="store", dest="add_source", default=False,
        #                    help="Add new source to LiquidSoap mixer [Experimental]")

        # playlist manipulation
        parser.add_argument("-fnp", "--fetch-new-programmes", action="store_true", dest="fetch_new_programme",
                            default=False, help="Fetch new programmes from calendarurl in comba.ini")
        parser.add_argument("-spe", "--swap-playlist-entries", action="store", dest="swap_playlist_entries", default=0,
                            metavar=("FROM", "TO"), nargs=2,
                            help="Swaps two Playlistentries")
        parser.add_argument("-dpe", "--delete-playlist-entry", action="store", dest="delete_playlist_entry", default=0,
                            metavar="INDEX",
                            help="Delete Playlistentry at INDEX")
        parser.add_argument("-ipe", "--insert-playlist-entry", action="store", dest="insert_playlist_entry", default=0,
                            metavar=("FROMTIME", "SOURCE"), nargs=2,
                            help="Add a new Playlistentry at a given index. Set fromtime with this format: 2017-12-31T13:30:00")  # , type=valid_playlist_entry)
        parser.add_argument("-pmq", "--print-message-queue", action="store_true", dest="print_message_queue",
                            default=False,
                            help="Prints message queue")

        # send a redis message
        parser.add_argument("-rm", "--redis-message", action="store", dest="redis_message", default=False,
                            metavar=("CHANNEL", "MESSAGE"), nargs=2,
                            help="Send a redis message to the Listeners")

        # calls from liquidsoap
        parser.add_argument("-gnf", "--get-next-file-for", action="store", dest="get_file_for", default=False,
                            metavar="PLAYLISTTYPE",
                            help="For which type you wanna GET a next audio file?")
        parser.add_argument("-snf", "--set-next-file-for", action="store", dest="set_file_for", default=False,
                            metavar=("PLAYLISTTYPE", "FILE"), nargs=2,
                            help="For which type you wanna SET a next audio file?")
        parser.add_argument("-np", "--now-playing", action="store_true", dest="now_playing", default=False,
                            help="Which source is now playing")
        parser.add_argument("-ip", "--init-player", action="store_true", dest="init_player", default=False,
                            help="Reset liquidsoap volume and mixer activations?")

        if len(sys.argv) == 1:
            raise ValueError("No Argument passed!")

        return parser


def valid_playlist_entry(argument):
    from datetime import datetime

    try:

        index = int(argument[0])
        fromtime = datetime.strptime(argument[1], "%Y-%m-%d")
        source = argument[2]
        return index, fromtime, source
    except:
        msg = "Not a valid date: '{0}'.".format(argument[0])
        raise


# # ## ## ## ## ## # #
# # ENTRY FUNCTION # #
# # ## ## ## ## ## # #
def main():
    Guru()
# # ## ## ## ## ## ## # #
# # End ENTRY FUNCTION # #
# # ## ## ## ## ## ## # #


if __name__ == "__main__":
    main()
