import sys
import time
import redis
import threading

from datetime import datetime
from threading import Event

from modules.communication.redis.messenger import RedisMessenger
from libraries.database.statestore import RedisStateStore
from libraries.exceptions.auraexceptions import RedisConnectionException
from libraries.enum.consolecolor import TerminalColors
from libraries.enum.redischannel import RedisChannel


class ServerRedisAdapter(threading.Thread, RedisMessenger):
    debug = False
    pubsub = None
    config = None
    redisdb = None
    channel = ""
#    auracontroller = None
    redisclient = None
    scheduler = None
    liquidsoapcommunicator = None

    def __init__(self):
        threading.Thread.__init__(self)
        RedisMessenger.__init__(self)

        # init
        threading.Thread.__init__ (self)
        self.shutdown_event = Event()

        self.channel = RedisChannel.STANDARD.value
        self.section = ''
        self.rstore = RedisStateStore()
        self.errnr = '00'
        self.components = {'controller':'01', 'scheduling':'02', 'playd':'03', 'recorder':'04', 'helpers':'09'}
        self.fromMail = ''
        self.adminMails = ''

        self.redisclient = ClientRedisAdapter()

    # ------------------------------------------------------------------------------------------ #
    def run(self):
        self.redisdb = redis.Redis()
        self.pubsub = self.redisdb.pubsub()
        self.pubsub.subscribe(self.channel)

        self.logger.info(TerminalColors.ORANGE.value + "waiting for REDIS message on channel " + self.channel + TerminalColors.ENDC.value)

        for item in self.pubsub.listen():
            if item["type"] == "subscribe":
                continue

            self.logger.info(TerminalColors.ORANGE.value + "received REDIS message: " + TerminalColors.ENDC.value + str(item))

            item["channel"] = self.decode_if_needed(item["channel"])
            item["data"] = self.decode_if_needed(item["data"])

            try:
                self.work(item)
            except RedisConnectionException as rce:
                self.logger.error(rce)

            self.logger.info(TerminalColors.ORANGE.value + "waiting for REDIS message on channel " + self.channel + TerminalColors.ENDC.value)

        self.pubsub.unsubscribe()
        self.logger.error("unsubscribed from " + self.channel + " and finished")

    # ------------------------------------------------------------------------------------------ #
    def decode_if_needed(self, val):
        if isinstance(val, bytes):
            return val.decode("utf-8")
        return val

    # ------------------------------------------------------------------------------------------ #
    def listen_for_one_message(self, channel, socket_timeout=2):
        self.redisdb = redis.Redis(socket_timeout=socket_timeout)
        self.pubsub = self.redisdb.pubsub()
        self.pubsub.subscribe(channel)

        try:
            self.logger.info("I am listening on channel '"+channel+"' for "+str(socket_timeout)+" seconds")

            for item in self.pubsub.listen():
                it = self.receive_message(item)
                if it is not None:
                    break
        except redis.exceptions.TimeoutError as te:
            raise te

        return item["data"]

    # ------------------------------------------------------------------------------------------ #
    def receive_message(self, item):
        if item["type"] == "subscribe":
            self.logger.info("i am subscribed to channel " + item["channel"].decode("utf-8"))
            return None

        item["channel"] = item["channel"].decode("utf-8")
        if isinstance(item["data"], bytes):
            item["data"] = item["data"].decode("utf-8")

        self.pubsub.unsubscribe()
        return item

    # ------------------------------------------------------------------------------------------ #
    def work(self, item):
        if item["data"] == "fetch_new_programme":
            self.execute(RedisChannel.FNP_REPLY.value, self.scheduler.fetch_new_programme, )

        elif item["data"] == "init_player":
            self.execute(RedisChannel.IP_REPLY.value, self.liquidsoapcommunicator.init_player)

        elif item["data"] == "get_act_programme":
            self.execute(RedisChannel.GAP_REPLY.value, self.scheduler.get_act_programme_as_string)

        elif item["data"] == "print_message_queue":
            self.execute(RedisChannel.PMQ_REPLY.value, self.scheduler.print_message_queue)

        elif item["data"].find("swap_playlist_entries") >= 0:
            extracted = item["data"].split()[1:3]
            param = {"from_index": extracted[0], "to_index": extracted[1]}
            self.execute(RedisChannel.MPE_REPLY.value, self.scheduler.swap_playlist_entries, param)

        elif item["data"].find("delete_playlist_entry") >= 0:
            entrynum = item["data"].split()[1]
            self.logger.info("entry to del: ", entrynum)
            self.execute(RedisChannel.DPE_REPLY.value, self.scheduler.delete_playlist_entry, entrynum)

        elif item["data"].find("insert_playlist_entry") >= 0:
            extracted = item["data"].split()[1:3]
            param = {"fromtime": extracted[0], "source": extracted[1]}
            self.execute(RedisChannel.IPE_REPLY.value, self.scheduler.insert_playlist_entry, param)

        elif item["data"].find("set_next_file") >= 0:
            playlist = item["data"].split()[1]
            playlist = playlist[0:len(playlist)-9]
            self.logger.critical("HAVE TO SET NEXT FILE FOR: "+playlist)
            self.execute(RedisChannel.SNF_REPLY.value, self.scheduler.set_next_file_for, playlist)

        else:
            raise RedisConnectionException("ServerRedisAdapter Cannot understand command: " + item["data"])

    # ------------------------------------------------------------------------------------------ #
    def execute(self, channel, f, param=None):
        if param:
            reply = f(param)
        else:
            reply = f()

        if reply is None:
            reply = ""

        # sometimes the sender is faster than the receiver. redis messages would be lost
        time.sleep(0.1)

        self.logger.info(TerminalColors.ORANGE.value + "replying " + reply + " on channel " + channel + TerminalColors.ENDC.value)

        # publish
        self.redisclient.publish(channel, reply)

    # ------------------------------------------------------------------------------------------ #
    def join_comm(self):
        try:
            while self.is_alive():
                self.logger.info(str(datetime.now())+" joining")
                self.join()
                self.logger.warning("join out")

        except (KeyboardInterrupt, SystemExit):
            # Dem Server den Shutdown event setzen
            # server.shutdown_event.set()
            # Der Server wartet auf Eingabe
            # Daher einen Client initiieren, der eine Nachricht schickt
            self.halt()
            sys.exit('Terminated')

    # ------------------------------------------------------------------------------------------ #
    def halt(self):
        """
        Stop the server
        """
        if self.shutdown_event.is_set():
            return
        try:
            del self.auracontroller
        except:
            pass
        self.shutdown_event.set()
        result = 'failed'
        try:
            result = self.socket.unbind("tcp://"+self.ip+":"+self.port)
        except:
            pass

        #self.socket.close()

    # ------------------------------------------------------------------------------------------ #
    def send(self, message):
        """
        Send a message to the client
        :param message: string
        """
        if not self.can_send:
            self.logger.info("sending a "+str(len(message))+" long message via REDIS.")
            self.socket.send(message.encode("utf-8"))
            self.can_send = False
        else:
            self.logger.warning("cannot send message via REDIS: "+str(message))

class ClientRedisAdapter(RedisMessenger):

    def __init__(self):
        RedisMessenger.__init__(self)

    # ------------------------------------------------------------------------------------------ #
    def publish(self, channel, message):
        if type(channel) == RedisChannel:
            channel = channel.value

        self.rstore.publish(channel, message)
